import React, { useEffect, useRef } from "react";
import { ROW, SW, NE, SE, NW, JOIN_RIGHT_ARROW, JOIN_UP_ARROW, JOIN_LEFT_ARROW, JOIN_DOWN_ARROW } from "../reducer/constants"; //import { replaceSubstitutionTransformer } from "common-tags";

export default function CornerOverlay(props) {
  var canvasRef = useRef(); //const propsRef = useRef({});
  // const shouldUpdate =
  //   propsRef.current.width === props.width &&
  //   propsRef.current.height === props.height;

  useEffect(function () {
    updateJoinOverlay();
    updateDivideOverlay();
  });

  function updateDivideOverlay() {
    var pane = props.pane,
        subdivide = props.subdivide;
    if (!pane.canSplit || !subdivide.cornerDown) return;
    var corner = subdivide.cornerDown.corner;
    var canvas = canvasRef.current;
    var ctx = canvas.getContext("2d");
    var width = pane.width,
        height = pane.height,
        top = pane.top,
        left = pane.left;
    height = Math.round(height + top - (top | 0));
    width = Math.round(width + left - (left | 0));
    var dashRatio = 0.5;
    var dashWidth = 3;
    var dashSpacing = 20;
    var dashLength = dashRatio * dashSpacing;
    var offset = 34;
    ctx.clearRect(0, 0, width, height);
    ctx.beginPath();
    ctx.rect(0, 0, width, height);

    if (corner === SW) {
      for (var x = 0; x < width; x += dashSpacing) {
        ctx.rect(x, height - offset - dashWidth, dashLength, dashWidth);
      }

      for (var y = 0; y < height; y += dashSpacing) {
        ctx.rect(offset, height - y - dashLength, dashWidth, dashLength);
      }
    } else if (corner === NE) {
      for (var _x = 0; _x < width; _x += dashSpacing) {
        ctx.rect(width - _x - dashLength, offset, dashLength, dashWidth);
      }

      for (var _y = 0; _y < height; _y += dashSpacing) {
        ctx.rect(width - offset - dashWidth, _y, dashWidth, dashLength);
      }
    } else if (corner === NW) {
      for (var _x2 = 0; _x2 < width; _x2 += dashSpacing) {
        ctx.rect(_x2, offset, dashLength, dashWidth);
      }

      for (var _y2 = 0; _y2 < height; _y2 += dashSpacing) {
        ctx.rect(offset, _y2, dashWidth, dashLength);
      }
    } else if (corner === SE) {
      for (var _x3 = 0; _x3 < width; _x3 += dashSpacing) {
        ctx.rect(width - _x3 - dashLength, height - offset - dashWidth, dashLength, dashWidth);
      }

      for (var _y3 = 0; _y3 < height; _y3 += dashSpacing) {
        ctx.rect(width - offset - dashWidth, height - _y3 - dashLength, dashWidth, dashLength);
      }
    }

    ctx.fillStyle = "#999";
    ctx.closePath();
    ctx.fill("evenodd");
  }

  function updateJoinOverlay() {
    var pane = props.pane;
    if (!pane.joinDirection) return;
    var canvas = canvasRef.current;
    var ctx = canvas.getContext("2d");
    var width = pane.width,
        height = pane.height,
        top = pane.top,
        left = pane.left,
        joinDirection = pane.joinDirection;
    height = Math.round(height + top - (top | 0));
    width = Math.round(width + left - (left | 0));
    var size = Math.min(width, height);
    var bodyHeight = size / 3 / 2 | 0;
    var bodyWidth = size / 3 / 2 | 0;
    var w2 = width / 2 | 0;
    var h2 = height / 2 | 0;
    ctx.clearRect(0, 0, width, height);
    ctx.beginPath();
    ctx.moveTo(0, 0);

    if (joinDirection === JOIN_RIGHT_ARROW) {
      ctx.lineTo(width, 0);
      ctx.lineTo(width, height);
      ctx.lineTo(0, height);
      ctx.lineTo(0, h2 + bodyHeight);
      ctx.lineTo(bodyWidth, h2 + bodyHeight);
      ctx.lineTo(bodyWidth, h2 + bodyHeight * 2);
      ctx.lineTo(size / 2, h2);
      ctx.lineTo(bodyWidth, h2 - bodyHeight * 2);
      ctx.lineTo(bodyWidth, h2 - bodyHeight);
      ctx.lineTo(0, h2 - bodyHeight);
    }

    if (joinDirection === JOIN_LEFT_ARROW) {
      ctx.lineTo(width, 0);
      ctx.lineTo(width, height);
      ctx.lineTo(width, h2 - bodyHeight);
      ctx.lineTo(width - bodyWidth, h2 - bodyHeight);
      ctx.lineTo(width - bodyWidth, h2 - bodyHeight * 2);
      ctx.lineTo(width - size / 2, h2);
      ctx.lineTo(width - bodyWidth, h2 + bodyHeight * 2);
      ctx.lineTo(width - bodyWidth, h2 + bodyHeight);
      ctx.lineTo(width, h2 + bodyHeight);
      ctx.lineTo(width, height);
      ctx.lineTo(0, height);
    }

    if (joinDirection === JOIN_UP_ARROW) {
      ctx.lineTo(0, height);
      ctx.lineTo(width, height);
      ctx.lineTo(w2 - bodyWidth, height);
      ctx.lineTo(w2 - bodyWidth, height - bodyHeight);
      ctx.lineTo(w2 - bodyWidth * 2, height - bodyHeight);
      ctx.lineTo(w2, height - size / 2);
      ctx.lineTo(w2 + bodyWidth * 2, height - bodyHeight);
      ctx.lineTo(w2 + bodyWidth, height - bodyHeight);
      ctx.lineTo(w2 + bodyWidth, height);
      ctx.lineTo(width, height);
      ctx.lineTo(width, 0);
    }

    if (joinDirection === JOIN_DOWN_ARROW) {
      ctx.lineTo(0, height);
      ctx.lineTo(width, height);
      ctx.lineTo(width, 0);
      ctx.lineTo(w2 + bodyWidth, 0);
      ctx.lineTo(w2 + bodyWidth, bodyHeight);
      ctx.lineTo(w2 + bodyWidth * 2, bodyHeight);
      ctx.lineTo(w2, size / 2);
      ctx.lineTo(w2 - bodyWidth * 2, bodyHeight);
      ctx.lineTo(w2 - bodyWidth, bodyHeight);
      ctx.lineTo(w2 - bodyWidth, 0);
    } //    ctx.lineWidth = 2


    ctx.fillStyle = "#999";
    ctx.closePath(); //ctx.stroke()

    ctx.fill();
  } // shouldComponentUpdate(nextProps) {
  //   return (
  //     this.props.width === nextProps.width &&
  //     this.props.height === nextProps.height
  //   );
  // }


  var subdivide = props.subdivide;
  var _props$pane = props.pane,
      joinDirection = _props$pane.joinDirection,
      canSplit = _props$pane.canSplit;
  if (!(subdivide.cornerDown || subdivide.dividerDown)) return false;

  if (!(joinDirection || canSplit)) {
    var dividerDown = subdivide.dividerDown;
    var cursor = !dividerDown ? undefined : dividerDown.direction === ROW ? "col-resize" : "row-resize";
    return React.createElement("div", {
      style: {
        width: "100%",
        height: "100%",
        position: "absolute",
        cursor: cursor,
        top: 0
      }
    });
  }

  var _props$pane2 = props.pane,
      width = _props$pane2.width,
      height = _props$pane2.height,
      top = _props$pane2.top,
      left = _props$pane2.left;
  height = Math.round(height + top - (top | 0));
  width = Math.round(width + left - (left | 0));
  return React.createElement("canvas", {
    width: width,
    height: height,
    style: {
      top: 0,
      left: 0,
      //  pointerEvents: "none",
      position: "absolute",
      background: "#fff",
      opacity: 0.9
    },
    ref: canvasRef
  });
}