import _objectSpread from "@babel/runtime/helpers/esm/objectSpread";
import React, { useEffect, useLayoutEffect, useRef, forwardRef, useCallback } from "react";
import { Pane } from "./Pane";
import { Dividers } from "./Dividers";
import AnimationFrame from "./AnimationFrame";
import { CHILD_ABOVE, CHILD_BELOW, CHILD_LEFT, CHILD_RIGHT, ROW, SW, NE, SE, NW } from "../reducer/constants";
import ResizeObserver from "resize-observer-polyfill";
export var Layout = forwardRef(function (props, ref) {
  var propsRef = useRef();
  var resizeEl = useRef();
  propsRef.current = props;
  var onResize = useCallback(function (entries) {
    var _entries$0$contentRec = entries[0].contentRect,
        width = _entries$0$contentRec.width,
        height = _entries$0$contentRec.height;
    props.actions.setSize(width, height);
  }, [resizeEl.current]);
  useLayoutEffect(function () {
    if (!resizeEl.current) {
      return;
    }

    var _resizeEl$current$get = resizeEl.current.getBoundingClientRect(),
        width = _resizeEl$current$get.width,
        height = _resizeEl$current$get.height;

    props.actions.setSize(width, height);
    var resizeObserver = new ResizeObserver(onResize);
    resizeObserver.observe(resizeEl.current);
    return function () {
      resizeObserver.unobserve(resizeEl.current);
    };
  }, [resizeEl.current]);
  useEffect(function () {
    //const props = propsRef.current;
    var animationFrame = new AnimationFrame();
    var onMouseMove = animationFrame.throttle(function (e) {
      var props = propsRef.current;
      var rect = resizeEl.current.getBoundingClientRect();
      var actions = props.actions,
          subdivide = props.store;
      var clientX = e.clientX - rect.x;
      var clientY = e.clientY - rect.y;

      if (subdivide.dividerDown) {
        e.preventDefault();
        var divider = subdivide.dividerDown;
        var beforePaneId = divider.beforePaneId,
            afterPaneId = divider.afterPaneId,
            direction = divider.direction,
            parentSize = divider.parentSize,
            startX = divider.startX,
            startY = divider.startY;
        var delta = direction === ROW ? clientX - startX : clientY - startY;
        var deltaRatio = delta / parentSize;
        var afterRatio = divider.afterRatio - deltaRatio;
        var beforeRatio = divider.beforeRatio + deltaRatio;

        if (beforeRatio * parentSize > 20 && afterRatio * parentSize > 20) {
          actions.setSplitRatios([{
            paneId: beforePaneId,
            ratio: beforeRatio
          }, {
            paneId: afterPaneId,
            ratio: afterRatio
          }]); //actions.setSplitRatio(beforePaneId, beforeRatio);
          //actions.setSplitRatio(afterPaneId, afterRatio);
        }
      }

      if (subdivide.cornerDown && !subdivide.dividerDown) {
        var pane = subdivide.cornerDown;
        var split = actions.split;
        var width = pane.width,
            height = pane.height,
            left = pane.left,
            top = pane.top,
            id = pane.id,
            corner = pane.corner;

        if (clientX > left && clientX < left + width && clientY > top && clientY < top + height) {
          if (corner === SW) {
            if (clientX - left > 25) {
              split(id, CHILD_LEFT, clientX, clientY);
            } else if (top + height - clientY > 25) {
              split(id, CHILD_BELOW, clientX, clientY);
            }
          }

          if (corner === NE) {
            if (left + width - clientX > 25) {
              split(id, CHILD_RIGHT, clientX, clientY);
            } else if (clientY - top > 25) {
              split(id, CHILD_ABOVE, clientX, clientY);
            }
          }

          if (corner === SE) {
            if (left + width - clientX > 25) {
              split(id, CHILD_RIGHT, clientX, clientY);
            } else if (top + height - clientY > 25) {
              split(id, CHILD_BELOW, clientX, clientY);
            }
          }

          if (corner === NW) {
            if (clientX - left > 25) {
              split(id, CHILD_LEFT, clientX, clientY);
            } else if (clientY - top > 25) {
              split(id, CHILD_ABOVE, clientX, clientY);
            }
          }
        }
      }
    }, []);

    function onMouseUp() {
      var props = propsRef.current;
      var actions = props.actions,
          subdivide = props.store;

      if (subdivide.dividerDown) {
        actions.setDividerDown(undefined);
      } // give pane onMouseUp a chance to fire


      setTimeout(function () {
        if (subdivide.cornerDown) {
          actions.setCornerDown(undefined);
        }
      }, 10);
    } // window.addEventListener("resize", () => {
    //   setSize(window.innerWidth, window.innerHeight);
    // });


    document.addEventListener("mouseup", onMouseUp);
    document.addEventListener("mousemove", onMouseMove); //setSize(window.innerWidth, window.innerHeight);

    return function () {
      animationFrame.stop();
      document.removeEventListener("mouseup", onMouseUp);
      document.removeEventListener("mousemove", onMouseMove);
    };
  }, []);
  var subdivide = props.store,
      actions = props.actions,
      DefaultComponent = props.DefaultComponent,
      iframeSafe = props.iframeSafe;
  var panes;

  if (iframeSafe) {
    panes = subdivide.allPanesIdsEver.map(function (id) {
      var pane = subdivide.panes[id];
      return React.createElement(Pane, {
        subdivide: subdivide,
        pane: pane,
        actions: actions,
        key: "pane" + id,
        DefaultComponent: DefaultComponent
      });
    });
  } else {
    panes = subdivide.panes.filter(function (pane) {
      return !pane.isGroup;
    }).map(function (pane) {
      return React.createElement(Pane, {
        subdivide: subdivide,
        pane: pane,
        actions: actions,
        key: pane.id,
        DefaultComponent: DefaultComponent
      });
    });
  }

  var _props$style = props.style,
      style = _props$style === void 0 ? {
    width: "100vw",
    height: "100vh"
  } : _props$style;
  return React.createElement("div", {
    ref: resizeEl,
    style: _objectSpread({}, style, {
      position: "relative"
    })
  }, panes, React.createElement(Dividers, {
    resizeEl: resizeEl,
    dividers: subdivide.dividers,
    subdivide: subdivide,
    actions: actions
  }));
});
Layout.defaultProps = {
  iframeSafe: true
};