import { ROW, COL, NE, SW, NW, SE, JOIN_RIGHT_ARROW, JOIN_UP_ARROW, JOIN_LEFT_ARROW, JOIN_DOWN_ARROW } from "./constants";

function getJoinDirection(state, pane) {
  var cornerDown = state.cornerDown;
  if (cornerDown === undefined) return false;
  var cornerDownId = state.cornerDown.id;
  var cornerDownPane = state.panes[cornerDownId];
  var parent = state.panes[cornerDownPane.parentId];
  if (!parent) return false;
  var siblings = parent.childIds;
  var index = siblings.indexOf(cornerDownId);
  var beforeId = index < 1 ? undefined : siblings[index - 1];
  var afterId = siblings[index + 1];
  var isBeforeGroup = beforeId !== undefined && state.panes[beforeId].isGroup;
  var isAfterGroup = afterId !== undefined && state.panes[afterId].isGroup;
  var canJoinBefore = beforeId === pane.id && !isBeforeGroup;
  var canJoinAfter = afterId === pane.id && !isAfterGroup;
  return cornerDown.corner === NE && (parent.direction === COL && canJoinBefore && JOIN_UP_ARROW || parent.direction === ROW && canJoinAfter && JOIN_RIGHT_ARROW) || cornerDown.corner === SW && (parent.direction === COL && canJoinAfter && JOIN_DOWN_ARROW || parent.direction === ROW && canJoinBefore && JOIN_LEFT_ARROW) || cornerDown.corner === NW && (parent.direction === COL && canJoinBefore && JOIN_UP_ARROW || parent.direction === ROW && canJoinBefore && JOIN_LEFT_ARROW) || cornerDown.corner === SE && (parent.direction === COL && canJoinAfter && JOIN_DOWN_ARROW || parent.direction === ROW && canJoinAfter && JOIN_RIGHT_ARROW);
}

export function secondPass(state) {
  var dividers = {};
  var dividerDownId = state.dividerDown && state.dividerDown.id || null;
  var rootId = state.rootId,
      width = state.width,
      height = state.height;
  var left = 0;
  var top = 0;
  var panes = state.panes;
  var rootPane = state.panes[rootId];
  var cellSpacing = state.cellSpacing,
      cornerDown = state.cornerDown;
  Object.assign(rootPane, {
    width: width,
    height: height,
    top: top,
    left: left,
    canSplit: cornerDown && cornerDown.id === rootId
  });
  panes.rootId = rootPane;

  function round(n, i) {
    return n === 0.5 ? i % 2 === 0 ? n - 0.5 : n + 0.5 : Math.round(n);
  }

  var flattenChildren = function flattenChildren(parent) {
    var x = parent.left;
    var y = parent.top;
    var spacingOffset;
    var hasDivider = false;
    var beforePaneId;
    var divider;
    var beforeRatio;
    var offcut = 0; //const kids = parent.childIds.map(id => panes[id]);
    // console.log(
    //   parent.width,
    //   kids.map(x => x.splitRatio),
    //   kids.map(x => x.splitRatio).reduce((a, b) => a + b, 0)
    // );

    parent.childIds.forEach(function (childId, i) {
      var child = panes[childId];
      var canSplit = cornerDown && cornerDown.id === childId;
      var joinDirection = getJoinDirection(state, child);
      Object.assign(child, {
        canSplit: canSplit,
        joinDirection: joinDirection
      });
      hasDivider = i !== 0;
      spacingOffset = 0;

      if (hasDivider) {
        spacingOffset = cellSpacing;
        divider = {
          left: x,
          top: y,
          beforePaneId: beforePaneId,
          afterPaneId: child.id,
          beforeRatio: beforeRatio,
          afterRatio: child.splitRatio,
          direction: parent.direction,
          parentSize: parent.direction === ROW ? parent.width : parent.height,
          id: beforePaneId + "n" + child.id
        };
      }

      if (parent.direction === ROW) {
        if (hasDivider) {
          divider.width = cellSpacing;
          divider.height = parent.height;
          dividers[divider.id] = divider;
          x += cellSpacing;
        }

        var pxAlign = parent.width * child.splitRatio - spacingOffset + (offcut < 0.5 ? offcut : -(1 - offcut));
        offcut = pxAlign % 1;
        var size = pxAlign + (offcut < 0.5 ? -offcut : 1 - offcut);
        Object.assign(child, {
          width: size,
          height: parent.height,
          left: x,
          top: y
        });
        x = x + size;
      } else if (parent.direction === COL) {
        if (hasDivider) {
          divider.width = parent.width;
          divider.height = cellSpacing;
          dividers[divider.id] = divider;
          y += cellSpacing;
        }

        var _pxAlign = parent.height * child.splitRatio - spacingOffset + (offcut < 0.5 ? offcut : -(1 - offcut));

        offcut = _pxAlign % 1;

        var _size = _pxAlign + (offcut < 0.5 ? -offcut : 1 - offcut);

        Object.assign(child, {
          width: parent.width,
          height: _size,
          left: x,
          top: y
        });
        y += _size;
      }

      beforePaneId = child.id;
      beforeRatio = child.splitRatio;

      if (child.isGroup) {
        flattenChildren(child);
      }
    });
  };

  flattenChildren(rootPane);
  state.dividers = dividers;
  state.didResize = false;
  return state;
}