import _objectSpread from "@babel/runtime/helpers/esm/objectSpread";
import { CHILD_ABOVE, CHILD_BELOW, CHILD_LEFT, CHILD_RIGHT, ROW, COL } from "./constants";
import { createPane } from "./createPane";
import { createLayout } from "./createLayout";
import { secondPass } from "./secondPass";

function getNextId(state) {
  var panes = state.panes,
      allPanesIdsEver = state.allPanesIdsEver;
  if (!allPanesIdsEver) return 1;
  return allPanesIdsEver[allPanesIdsEver.length - 1] * 1 + 1;
  var id = 0;

  while (panes["" + id] !== undefined) {
    id += 1;
  }

  return "" + id;
}

function getDirection(splitType) {
  if (splitType === CHILD_ABOVE || splitType === CHILD_BELOW) return COL;
  if (splitType === CHILD_LEFT || splitType === CHILD_RIGHT) return ROW;
}

function getOffset(splitType) {
  if (splitType === CHILD_ABOVE || splitType === CHILD_LEFT) return 0;
  if (splitType === CHILD_BELOW || splitType === CHILD_RIGHT) return 1;
}

export function setSplitRatio(state, _ref) {
  var splitRatio = _ref.splitRatio,
      id = _ref.id;
  state.panes[id].splitRatio = splitRatio;
  return state;
}
export function setSplitRatios(state, _ref2) {
  var panes = _ref2.panes;
  panes.forEach(function (_ref3) {
    var paneId = _ref3.paneId,
        ratio = _ref3.ratio;
    state.panes[paneId].splitRatio = ratio;
  });
  return state;
} // export function moveDivider(state, { x, y }) {
//   const divider = state.dividerDown;
//   const {
//     beforePaneId,
//     afterPaneId,
//     direction,
//     parentSize,
//     startX,
//     startY
//   } = divider;
//   const delta = direction === ROW ? x - startX : y - startY;
//   const deltaRatio = delta / parentSize;
//   const afterRatio = divider.afterRatio - deltaRatio;
//   const beforeRatio = divider.beforeRatio + deltaRatio;
//   if (beforeRatio * parentSize > 20 && afterRatio * parentSize > 20) {
//     state.panes[beforePaneId].splitRatio = beforeRatio;
//     state.panes[afterPaneId].splitRatio = afterRatio;
//   }
//   return state;
// }

export function setDividerStyles(state, _ref4) {
  var borderColor = _ref4.borderColor,
      cellSpaceColor = _ref4.cellSpaceColor,
      borderSize = _ref4.borderSize,
      cellSpacing = _ref4.cellSpacing,
      touchMargin = _ref4.touchMargin;
  if (borderColor !== undefined) state.borderColor = borderColor;
  if (cellSpaceColor !== undefined) state.cellSpaceColor = cellSpaceColor;
  if (borderSize !== undefined) state.borderSize = borderSize;
  if (cellSpacing !== undefined) state.cellSpacing = cellSpacing;
  if (touchMargin !== undefined) state.touchMargin = touchMargin;
  return state;
}
export function setSize(state, _ref5) {
  var width = _ref5.width,
      height = _ref5.height;
  state.width = width;
  state.height = height;
  return state;
}
export function setCornerDown(state, _ref6) {
  var cornerDown = _ref6.cornerDown;
  state.cornerDown = cornerDown;
  return state;
}
export function setCornerHover(state, _ref7) {
  var cornerHover = _ref7.cornerHover;
  state.cornerHover = cornerHover;
  return state;
}
export function setDividerDown(state, _ref8) {
  var divider = _ref8.divider;
  state.dividerDown = divider;
  if (!divider) return;
  divider.beforeRatio = state.panes[divider.beforePaneId].splitRatio;
  divider.afterRatio = state.panes[divider.afterPaneId].splitRatio;
  return state;
}
export function setPaneProps(state, _ref9) {
  var id = _ref9.id,
      props = _ref9.props;
  state.panes[id].props = _objectSpread({}, props);
  return state;
}

function wrapPane(state, id) {
  var panes = state.panes;
  var pane = panes[id];
  var parentId = pane.parentId,
      splitRatio = pane.splitRatio;
  var parent = panes[parentId];
  var groupId = getNextId(state);
  var group = createPane({
    id: groupId,
    isGroup: true,
    childIds: [id],
    parentId: parentId,
    splitRatio: splitRatio
  });
  state.allPanesIdsEver.push(groupId);
  pane.parentId = groupId;
  panes[groupId] = group;

  if (parent) {
    var childIds = parent.childIds;
    childIds[childIds.indexOf(id)] = groupId;
  }

  return {
    state: state,
    group: group
  };
}

function removePane(state, id) {
  //splice pane out of parents childIds
  var panes = state.panes;
  var pane = panes[id];
  var parent = panes[pane.parentId];
  if (!parent) return state;
  var childIds = parent.childIds;
  childIds.splice(childIds.indexOf(id), 1);

  if (childIds.length === 1) {
    var remainingPane = panes[childIds[0]];

    if (parent.id === state.rootId) {
      state.rootId = remainingPane.id;
      remainingPane.parentId = undefined;
    } else {
      var grandparentId = parent.parentId;
      var grandparent = panes[grandparentId];
      var grandchildIds = grandparent.childIds;
      grandchildIds[grandchildIds.indexOf(parent.id)] = remainingPane.id;
      remainingPane.parentId = grandparentId;
    }

    delete panes[parent.id];
    remainingPane.direction = undefined;
  }

  delete panes[id];
  return state;
}

export function split(state, _ref10) {
  var id = _ref10.id,
      splitType = _ref10.splitType,
      startX = _ref10.startX,
      startY = _ref10.startY;
  if (state.cornerDown === undefined) return;
  var panes = state.panes,
      rootId = state.rootId;
  var pane = panes[id];
  var oldPane = pane;
  var oldParentId = pane.parentId;
  var parent = panes[oldParentId];
  var isRoot = id === rootId;
  var direction = getDirection(splitType);

  if (!parent || parent.direction !== direction) {
    var _wrapPane = wrapPane(state, id),
        group = _wrapPane.group;

    parent = group;
    parent.direction = direction; // state = wrappedState.state;

    if (isRoot) {
      state.rootId = parent.id;
    }

    pane.splitRatio = 1;
  }

  var _parent = parent,
      childIds = _parent.childIds;
  var index = childIds.indexOf(id);
  var newPane = createPane({
    id: getNextId(state),
    parentId: parent.id,
    splitRatio: 0.2
  });
  var offset = getOffset(splitType);
  childIds.splice(index + offset, 0, newPane.id);
  var beforePaneId = offset === 1 ? pane.id : newPane.id;
  var afterPaneId = offset === 1 ? newPane.id : pane.id;
  var ratio = direction === ROW ? (startX - oldPane.left) / oldPane.width : (startY - oldPane.top) / oldPane.height;
  ratio = offset ? ratio : 1 - ratio;
  var ratioA = ratio;
  var ratioB = 1 - ratioA;

  if (newPane.parentId === oldParentId) {
    ratioA *= oldPane.splitRatio;
    ratioB *= oldPane.splitRatio;
  }

  pane.splitRatio = ratioA;
  newPane.splitRatio = ratioB;
  state.allPanesIdsEver.push(newPane.id);
  panes[parent.id] = parent;
  panes[newPane.id] = newPane;
  state.cornerDown = undefined;
  secondPass(state);
  state.dividerDown = _objectSpread({}, state.dividers["".concat(beforePaneId, "n").concat(afterPaneId)], {
    startX: startX,
    startY: startY
  });
  return state;
}
export function join(state, _ref11) {
  var retainId = _ref11.retainId,
      removeId = _ref11.removeId;
  var remove = state.panes[removeId]; // concurrent react is calling this twice for a single remove

  if (remove === undefined || remove.isGroup) {
    return state;
  }

  var retain = state.panes[retainId];
  var parent = state.panes[retain.parentId];
  var siblings = parent.childIds;
  var pos = [retainId, removeId].map(function (id) {
    return siblings.indexOf(id);
  });

  if (pos[1] === -1 || pos[0] === -1 || !(pos[0] + 1 === pos[1] || pos[0] - 1 === pos[1])) {
    console.warn("pane must be adjacent to join");
    return state;
  }

  removePane(state, removeId);
  var nextParentId = state.panes[retainId].parentId;
  var splitRatio = parent.id === nextParentId ? remove.splitRatio + retain.splitRatio : parent.splitRatio;
  state.panes[retain.id].splitRatio = splitRatio;
  return state;
}